==> ponyc-0.13.0-1-x86_64.pkg.tar.xz: usr/share/doc/pony/stdlib-docs/docs/process-ProcessMonitor.md <==
# ProcessMonitor

Forks and monitors a process. Notifies a client about STDOUT / STDERR events.


```pony
actor tag ProcessMonitor
```

## Constructors

### create

```pony
new tag create(
  auth: (AmbientAuth val | StartProcessAuth val),
  notifier: ProcessNotify iso,
  filepath: FilePath val,
  args: Array[String val] val,
  vars: Array[String val] val)
: ProcessMonitor tag^
```
#### Parameters

*   auth: ([AmbientAuth](builtin-AmbientAuth) val | [StartProcessAuth](process-StartProcessAuth) val)
*   notifier: [ProcessNotify](process-ProcessNotify) iso
*   filepath: [FilePath](files-FilePath) val
*   args: [Array](builtin-Array)\[[String](builtin-String) val\] val
*   vars: [Array](builtin-Array)\[[String](builtin-String) val\] val

#### Returns

* [ProcessMonitor](process-ProcessMonitor) tag^

---

## Public Behaviours

### print

Print some bytes and insert a newline afterwards.


```pony
be print(
  data: (String val | Array[U8 val] val))
```
#### Parameters

*   data: ([String](builtin-String) val | [Array](builtin-Array)\[[U8](builtin-U8) val\] val)

---

### write

Write to STDIN of the child process.
TODO: Signal back success/failure


```pony
be write(
  data: (String val | Array[U8 val] val))
```
#### Parameters

*   data: ([String](builtin-String) val | [Array](builtin-Array)\[[U8](builtin-U8) val\] val)

---

### printv

Print an iterable collection of ByteSeqs.


```pony
be printv(
  data: ByteSeqIter val)
```
#### Parameters

*   data: [ByteSeqIter](builtin-ByteSeqIter) val

---

### writev

Write an iterable collection of ByteSeqs.


```pony
be writev(
  data: ByteSeqIter val)
```
#### Parameters

*   data: [ByteSeqIter](builtin-ByteSeqIter) val

---

### done_writing

Close _stdin_write file descriptor.


```pony
be done_writing()
```

---

### dispose

Terminate child and close down everything.


```pony
be dispose()
```

---

## Public Functions

### expect

A `stdout` call on the notifier must contain exactly `qty` bytes. If
`qty` is zero, the call can contain any amount of data.


```pony
fun ref expect(
  qty: USize val = 0)
: None val
```
#### Parameters

*   qty: [USize](builtin-USize) val = 0

#### Returns

* [None](builtin-None) val

---

## Private Behaviours

### _event_notify

Handle the incoming event.


```pony
be _event_notify(
  event: Pointer[AsioEvent val] tag,
  flags: U32 val,
  arg: U32 val)
```
#### Parameters

*   event: [Pointer](builtin-Pointer)\[[AsioEvent](builtin-AsioEvent) val\] tag
*   flags: [U32](builtin-U32) val
*   arg: [U32](builtin-U32) val

---

### _read_again

Resume reading on file descriptor.


```pony
be _read_again(
  fd: U32 val)
```
#### Parameters

*   fd: [U32](builtin-U32) val

---

## Private Functions

### _child

We are now in the child process. We redirect STDIN, STDOUT and STDERR
to their pipes and execute the command. The command is executed via
execve which does not return on success, and the text, data, bss, and
stack of the calling process are overwritten by that of the program
loaded. We've set the FD_CLOEXEC flag on all file descriptors to ensure
that they are all closed automatically once @execve gets called.


```pony
fun box _child(
  path: String val,
  argp: Array[Pointer[U8 val] tag] ref,
  envp: Array[Pointer[U8 val] tag] ref)
: None val
```
#### Parameters

*   path: [String](builtin-String) val
*   argp: [Array](builtin-Array)\[[Pointer](builtin-Pointer)\[[U8](builtin-U8) val\] tag\] ref
*   envp: [Array](builtin-Array)\[[Pointer](builtin-Pointer)\[[U8](builtin-U8) val\] tag\] ref

#### Returns

* [None](builtin-None) val

---

### _parent

We're now in the parent process. We setup asio events for STDOUT and STDERR
and close the file descriptors we don't need.


```pony
fun ref _parent()
: None val
```

#### Returns

* [None](builtin-None) val

---

### _make_argv

Convert an array of String parameters into an array of
C pointers to same strings.


```pony
fun box _make_argv(
  args: Array[String val] box)
: Array[Pointer[U8 val] tag] ref
```
#### Parameters

*   args: [Array](builtin-Array)\[[String](builtin-String) val\] box

#### Returns

* [Array](builtin-Array)\[[Pointer](builtin-Pointer)\[[U8](builtin-U8) val\] tag\] ref

---

### _dup2

Creates a copy of the file descriptor oldfd using the file
descriptor number specified in newfd. If the file descriptor newfd
was previously open, it is silently closed before being reused.
If dup2() fails because of EINTR we retry.


```pony
fun box _dup2(
  oldfd: U32 val,
  newfd: U32 val)
: None val
```
#### Parameters

*   oldfd: [U32](builtin-U32) val
*   newfd: [U32](builtin-U32) val

#### Returns

* [None](builtin-None) val

---

### _make_pipe

Creates a pipe, an unidirectional data channel that can be used
for interprocess communication. We need to set the flags O_NONBLOCK
and O_CLOEXEC on the two file descriptors here to prevent capturing
by another thread.


```pony
fun box _make_pipe(
  fd_flags: I32 val)
: (U32 val , U32 val) ?
```
#### Parameters

*   fd_flags: [I32](builtin-I32) val

#### Returns

* ([U32](builtin-U32) val , [U32](builtin-U32) val) ?

---

### _getflags

Get the string representation of flags set for a file descriptor


```pony
fun box _getflags(
  fd: U32 val)
: String box ?
```
#### Parameters

*   fd: [U32](builtin-U32) val

#### Returns

* [String](builtin-String) box ?

---

### _set_fd

```pony
fun box _set_fd(
  fd: U32 val,
  flags: I32 val)
: None val ?
```
#### Parameters

*   fd: [U32](builtin-U32) val
*   flags: [I32](builtin-I32) val

#### Returns

* [None](builtin-None) val ?

---

### _set_fl

```pony
fun box _set_fl(
  fd: U32 val,
  flags: I32 val)
: None val ?
```
#### Parameters

*   fd: [U32](builtin-U32) val
*   flags: [I32](builtin-I32) val

#### Returns

* [None](builtin-None) val ?

---

### _print_final

```pony
fun ref _print_final(
  data: (String val | Array[U8 val] val))
: None val
```
#### Parameters

*   data: ([String](builtin-String) val | [Array](builtin-Array)\[[U8](builtin-U8) val\] val)

#### Returns

* [None](builtin-None) val

---

### _write_final

```pony
fun ref _write_final(
  data: (String val | Array[U8 val] val))
: None val
```
#### Parameters

*   data: ([String](builtin-String) val | [Array](builtin-Array)\[[U8](builtin-U8) val\] val)

#### Returns

* [None](builtin-None) val

---

### _kill_child

Terminate the child process.


```pony
fun box _kill_child()
: None val ?
```

#### Returns

* [None](builtin-None) val ?

---

### _create_asio_event

Takes a file descriptor (one end of a pipe) and returns an AsioEvent.


```pony
fun box _create_asio_event(
  fd: U32 val)
: Pointer[AsioEvent val] tag
```
#### Parameters

*   fd: [U32](builtin-U32) val

#### Returns

* [Pointer](builtin-Pointer)\[[AsioEvent](builtin-AsioEvent) val\] tag

---

### _event_flags

Return all flags of an event as a string.


```pony
fun box _event_flags(
  flags: U32 val)
: String box
```
#### Parameters

*   flags: [U32](builtin-U32) val

#### Returns

* [String](builtin-String) box

---

### _close_fd

Close a file descriptor.


```pony
fun ref _close_fd(
  fd: U32 val)
: None val
```
#### Parameters

*   fd: [U32](builtin-U32) val

#### Returns

* [None](builtin-None) val

---

### _close

Close all pipes, unsubscribe events and wait for the child process to exit.


```pony
fun ref _close()
: None val
```

#### Returns

* [None](builtin-None) val

---

### _try_shutdown

If neither stdout nor stderr are open we close down and exit.


```pony
fun ref _try_shutdown()
: None val
```

#### Returns

* [None](builtin-None) val

---

### _pending_reads

Read from stdout while data is available. If we read 4 kb of data,
send ourself a resume message and stop reading, to avoid starving
other actors.
It's safe to use the same buffer for stdout and stderr because of
causal messaging. Events get processed one _after_ another.


```pony
fun ref _pending_reads(
  fd: U32 val)
: Bool val
```
#### Parameters

*   fd: [U32](builtin-U32) val

#### Returns

* [Bool](builtin-Bool) val

---

### _read_buf_size

```pony
fun ref _read_buf_size()
: None val
```

#### Returns

* [None](builtin-None) val

---

==> ponyc-0.18.1-1-x86_64.pkg.tar.xz: usr/share/doc/pony/stdlib-docs/docs/process-ProcessMonitor.md <==
# ProcessMonitor

Forks and monitors a process. Notifies a client about STDOUT / STDERR events.


```pony
actor tag ProcessMonitor
```

## Constructors

### create

```pony
new tag create(
  auth: (AmbientAuth val | StartProcessAuth val),
  notifier: ProcessNotify iso,
  filepath: FilePath val,
  args: Array[String val] val,
  vars: Array[String val] val)
: ProcessMonitor tag^
```
#### Parameters

*   auth: ([AmbientAuth](builtin-AmbientAuth) val | [StartProcessAuth](process-StartProcessAuth) val)
*   notifier: [ProcessNotify](process-ProcessNotify) iso
*   filepath: [FilePath](files-FilePath) val
*   args: [Array](builtin-Array)\[[String](builtin-String) val\] val
*   vars: [Array](builtin-Array)\[[String](builtin-String) val\] val

#### Returns

* [ProcessMonitor](process-ProcessMonitor) tag^

---

## Public Behaviours

### print

Print some bytes and append a newline.


```pony
be print(
  data: (String val | Array[U8 val] val))
```
#### Parameters

*   data: ([String](builtin-String) val | [Array](builtin-Array)\[[U8](builtin-U8) val\] val)

---

### write

Write to STDIN of the child process.


```pony
be write(
  data: (String val | Array[U8 val] val))
```
#### Parameters

*   data: ([String](builtin-String) val | [Array](builtin-Array)\[[U8](builtin-U8) val\] val)

---

### printv

Print an iterable collection of ByteSeqs.


```pony
be printv(
  data: ByteSeqIter val)
```
#### Parameters

*   data: [ByteSeqIter](builtin-ByteSeqIter) val

---

### writev

Write an iterable collection of ByteSeqs.


```pony
be writev(
  data: ByteSeqIter val)
```
#### Parameters

*   data: [ByteSeqIter](builtin-ByteSeqIter) val

---

### done_writing

Set the _done_writing flag to true. If the _pending is empty we can
close the _stdin_write file descriptor.


```pony
be done_writing()
```

---

### dispose

Terminate child and close down everything.


```pony
be dispose()
```

---

## Public Functions

### expect

A `stdout` call on the notifier must contain exactly `qty` bytes. If
`qty` is zero, the call can contain any amount of data.


```pony
fun ref expect(
  qty: USize val = 0)
: None val
```
#### Parameters

*   qty: [USize](builtin-USize) val = 0

#### Returns

* [None](builtin-None) val

---

## Private Behaviours

### _event_notify

Handle the incoming event.


```pony
be _event_notify(
  event: Pointer[AsioEvent val] tag,
  flags: U32 val,
  arg: U32 val)
```
#### Parameters

*   event: [Pointer](builtin-Pointer)\[[AsioEvent](builtin-AsioEvent) val\] tag
*   flags: [U32](builtin-U32) val
*   arg: [U32](builtin-U32) val

---

### _read_again

Resume reading on file descriptor.


```pony
be _read_again(
  fd: U32 val)
```
#### Parameters

*   fd: [U32](builtin-U32) val

---

## Private Functions

### _child

We are now in the child process. We redirect STDIN, STDOUT and STDERR
to their pipes and execute the command. The command is executed via
execve which does not return on success, and the text, data, bss, and
stack of the calling process are overwritten by that of the program
loaded. We've set the FD_CLOEXEC flag on all file descriptors to ensure
that they are all closed automatically once @execve gets called.


```pony
fun box _child(
  path: String val,
  argp: Array[Pointer[U8 val] tag] ref,
  envp: Array[Pointer[U8 val] tag] ref)
: None val
```
#### Parameters

*   path: [String](builtin-String) val
*   argp: [Array](builtin-Array)\[[Pointer](builtin-Pointer)\[[U8](builtin-U8) val\] tag\] ref
*   envp: [Array](builtin-Array)\[[Pointer](builtin-Pointer)\[[U8](builtin-U8) val\] tag\] ref

#### Returns

* [None](builtin-None) val

---

### _parent

We're now in the parent process. We setup asio events for STDOUT and STDERR
and close the file descriptors we don't need.


```pony
fun ref _parent()
: None val
```

#### Returns

* [None](builtin-None) val

---

### _make_argv

Convert an array of String parameters into an array of
C pointers to same strings.


```pony
fun box _make_argv(
  args: Array[String val] box)
: Array[Pointer[U8 val] tag] ref
```
#### Parameters

*   args: [Array](builtin-Array)\[[String](builtin-String) val\] box

#### Returns

* [Array](builtin-Array)\[[Pointer](builtin-Pointer)\[[U8](builtin-U8) val\] tag\] ref

---

### _dup2

Creates a copy of the file descriptor oldfd using the file
descriptor number specified in newfd. If the file descriptor newfd
was previously open, it is silently closed before being reused.
If dup2() fails because of EINTR we retry.


```pony
fun box _dup2(
  oldfd: U32 val,
  newfd: U32 val)
: None val
```
#### Parameters

*   oldfd: [U32](builtin-U32) val
*   newfd: [U32](builtin-U32) val

#### Returns

* [None](builtin-None) val

---

### _make_pipe

Creates a pipe, an unidirectional data channel that can be used
for interprocess communication. We need to set the flag on the
two file descriptors here to prevent capturing by another thread.


```pony
fun box _make_pipe(
  fd_flags: I32 val)
: (U32 val , U32 val) ?
```
#### Parameters

*   fd_flags: [I32](builtin-I32) val

#### Returns

* ([U32](builtin-U32) val , [U32](builtin-U32) val) ?

---

### _getflags

Get the string representation of flags set for a file descriptor


```pony
fun box _getflags(
  fd: U32 val)
: String box ?
```
#### Parameters

*   fd: [U32](builtin-U32) val

#### Returns

* [String](builtin-String) box ?

---

### _set_fd

```pony
fun box _set_fd(
  fd: U32 val,
  flags: I32 val)
: None val ?
```
#### Parameters

*   fd: [U32](builtin-U32) val
*   flags: [I32](builtin-I32) val

#### Returns

* [None](builtin-None) val ?

---

### _set_fl

```pony
fun box _set_fl(
  fd: U32 val,
  flags: I32 val)
: None val ?
```
#### Parameters

*   fd: [U32](builtin-U32) val
*   flags: [I32](builtin-I32) val

#### Returns

* [None](builtin-None) val ?

---

### _print_final

```pony
fun ref _print_final(
  data: (String val | Array[U8 val] val))
: None val
```
#### Parameters

*   data: ([String](builtin-String) val | [Array](builtin-Array)\[[U8](builtin-U8) val\] val)

#### Returns

* [None](builtin-None) val

---

### _kill_child

Terminate the child process.


```pony
fun box _kill_child()
: None val ?
```

#### Returns

* [None](builtin-None) val ?

---

### _event_flags

Return all flags of an event as a string.


```pony
fun box _event_flags(
  flags: U32 val)
: String box
```
#### Parameters

*   flags: [U32](builtin-U32) val

#### Returns

* [String](builtin-String) box

---

### _close_fd

Close a file descriptor.


```pony
fun ref _close_fd(
  fd: U32 val)
: None val
```
#### Parameters

*   fd: [U32](builtin-U32) val

#### Returns

* [None](builtin-None) val

---

### _close

Close all pipes, unsubscribe events and wait for the child process to exit.


```pony
fun ref _close()
: None val
```

#### Returns

* [None](builtin-None) val

---

### _try_shutdown

If neither stdout nor stderr are open we close down and exit.


```pony
fun ref _try_shutdown()
: None val
```

#### Returns

* [None](builtin-None) val

---

### _pending_reads

Read from stdout while data is available. If we read 4 kb of data,
send ourself a resume message and stop reading, to avoid starving
other actors.
It's safe to use the same buffer for stdout and stderr because of
causal messaging. Events get processed one _after_ another.


```pony
fun ref _pending_reads(
  fd: U32 val)
: Bool val
```
#### Parameters

*   fd: [U32](builtin-U32) val

#### Returns

* [Bool](builtin-Bool) val

---

### _read_buf_size

```pony
fun ref _read_buf_size()
: None val
```

#### Returns

* [None](builtin-None) val

---

### _write_final

Write as much as possible to the fd. Return false if not
everything was written.


```pony
fun ref _write_final(
  data: (String val | Array[U8 val] val))
: Bool val
```
#### Parameters

*   data: ([String](builtin-String) val | [Array](builtin-Array)\[[U8](builtin-U8) val\] val)

#### Returns

* [Bool](builtin-Bool) val

---

### _pending_writes

Send pending data. If any data can't be sent, keep it and mark fd as not
writeable. Once set to false the _stdin_writeable flag can only be cleared
here by processing the pending writes. If the _done_writing flag is set we
close the _stdin_write fd once we've processed pending writes.


```pony
fun ref _pending_writes()
: Bool val
```

#### Returns

* [Bool](builtin-Bool) val

---
